REBOL [
	Title:    "Berkeley DB accessor library"
	Purpose: {
		Provides bindings to Berkeley DB 1.85
		through companion C interface library 
		(libdb.c). 
    }
	Copyright: "All rights reserved."
	Author:    "Jeff Kreis"
	Email:     jeff@rebol.com
	Date:      15-Jan-2002
]

;- Determine platform
switch/default system/version * 0.0.0.1.1 [
	0.0.0.3.1 [
		platform: 'windows
		dblib-f: %libdb.dll
	]
][	
	platform: 'unix
	dblib-f: %libdb.so
]

dbso: load/library dblib-f

DBT: do compose [
	make (
		first dbts: [
			[
				struct! [
					data [string!]
					size [int]
				]
			]
		]
	) none
]

;;-- Both of these make our
;;   key and data DBT, but you might
;;   change make-key-dbt to something
;;   more specific
make-key-dbt: make-dbt: func [str [any-string! date!] /local len][
	str: to-string str
	len: length? str
	make struct! first dbt reduce [str len]
]

;;-- Make all the routines
foreach [r a f] compose/deep [
	get-flags [
		return: [
			struct! [
				creat    [int]
				excl     [int]
				exlock   [int]
				nonblock [int]
				rdonly   [int]
				rdwr     [int]
				shlock   [int]
				trunc    [int]
				binary   [int]
			]
		]		
	] "dbflg"
	get-types [
		return: [
			struct! [
				btree    [int]
				hash     [int]
				recno    [int]
			]
		]
	] "dbtyp"
	get-curse [
		return: [
			struct! [
				cursor      [int]
				iafter      [int]
				ibefore	    [int]
				nooverwrite [int]
				setcursor	[int]
				first	    [int]
				last	    [int]
				next	    [int]
				prev        [int]
			]
		] 
	]"dbcrs"
	db-open [
		file  [string!]
		flags [integer!]
		mode  [integer!]
		type  [integer!]
		return: [int] "DB*"
	] "rdb_open"
	db-close [db [int] return: [int]] "rdb_close"
	db-fd    [db [int] return: [int]] "rdb_fd"
	db-del   [
		db [int] key (dbts) 
		flags [int] return: [int]
	] "rdb_del"
	db-get   [
		db [int] key (dbts) data (dbts) 
		flags [int] return: [int]
	] "rdb_get"	
	db-bget-size [
		db [int] key (dbts) return: [int]
	] "rdb_bget_size"
;-- See README.txt about the binary! issue.
;	db-bget [data [binary!]] "rdb_bget"
	db-bget [data [string!]] "rdb_bget"
	db-put   [
		db [int] key (dbts) data (dbts) 
		flags [int] return: [int]
	] "rdb_put"	
	db-seq   [
		db [int] key (dbts) data (dbts) 
		flags [int] return: [int]
	] "rdb_seq"	
	db-sync [
		db [int] flags [int] return: [int]
	] "rdb_sync"
	db-rec-open [
		name [string!] flags [int] mode [int] return: [int]
	] "rdb_recno_open"
	db-rec-put [
		db [int] loc [double] data (dbts) flags [int] return: [int]
	] "rdb_recno_put"
	db-rec-get [
		db [int] loc [double] data (dbts) flags [int] return: [int]
	] "rdb_recno_get"
][set r make routine! a dbso f]

db-flags:  get-flags
db-types:  get-types
db-cursor: get-curse

db-do-open: func [name [string!] type [integer!]][
	db-open name db-flags/rdwr or db-flags/creat or db-flags/binary 436 type
]

dbbtree-open: func [name [string!]][db-do-open name db-types/btree]
dbhash-open:  func [name [string!]][db-do-open name db-types/hash]
dbrecno-open: func [name [string!]][
	db-rec-open name db-flags/rdwr or db-flags/creat 436
]

rec-put: func [db [integer!] n [number!] data [any-string!] /local dt][
	if n < 0 [make error! reform ["Bad record number!:" n]]
	dt: make-dbt replace/all data "^(null)" #
	n: to-decimal n
	db-rec-put db n dt 0 
]

rec-get: func [db [integer!] n [number!] /local dt rs][
	if n < 0 [make error! reform ["Bad record number!:" n]]
	dt: make-dbt copy ""
	n: to-decimal n
	either 0 <> rs: db-rec-get db n dt 0 [
		print ["woops.." mold dt]
		make error! reform ["REC-GET: Bad return val:" rs]
	][dt]
]

;;-- Gets a binary record
;;   See rdb_bget in libdb.c
dbgt-b: func [
	db [integer!]
	id [any-string! date!] 
	/locat dt dk sz err
][
	dk: make-key-dbt id

	if -1 <> sz: db-bget-size db dk [
		;;-- The latest, not yet released
		;;   version of Command, allows binary!
		;;   in struct values and passing
		;;   of binary where char* data is
		;;   expected. (See README.txt)
		
;		dk: make binary! sz
		dk: make string! sz
		insert/dup dk #"^(null)" sz
		db-bget dk
		to-binary dk
	] ;- none otherwise
]

dbgt: func [
	db [integer!] 
	id [date! any-string!] 
	/local dt dk
][
	dk: make-key-dbt id
	dt: make-dbt ""
	either 0 <> db-get db dk dt 0 [
		none
	][dt]
]

dbpt: func [
	db   [integer!] 
	key  [date! any-string!]
	data [any-string!]
	/local dt dk dbb err
][
	dk: make-key-dbt key
	dt: make-dbt data
	db-put db dk dt 0
]
