REBOL [
	Title: "DB lib tester"
]

do %db.r

dbg-prin:  func [x][prin x]
dbg-print: func [x][print x]

test-data: parse system/license none
replace/all test-data "" []

mk10: func [/local z][
	z: copy ""
	loop 10 [append z random/only "abcdefghijklmnopqrstuvwxyz"]
]

tst-01: does [
	dbg-print "Btree and hash binary put/get test"
	foreach [dbname dbop][
		"btree.db"   dbbtree-open
		"hash.db"    dbhash-open
	][
		error? try [delete to-file dbname]
		dbop: get dbop
		db: dbop dbname
		i: 0
		foreach v test-data [
			k: to-binary i
			dbpt db k compress v
			tv: dbgt-b db k
			if v <> tv: decompress tv [
				print ["Mismatch:" v tv]
			]
			i: i + 1
			if 0 = (i // 50) [prin "."]
		]
		db-close db 0
		print [newline dbname "=> passed"]
	]
]

tst-02: does [
	dbg-print "Big memory test"
	foreach [dbname dbop][
		"btree.db"   dbbtree-open
		"hash.db"    dbhash-open
	][
		error? try [delete to-file dbname]
		dbop: get dbop
		db: dbop dbname

		repeat j 1000 [
			a: mk10 
			b: mk10
			
			if 0 = (j // 100) [
				print ["#" j "REBOL memory:" system/stats]
			]
			loop 100 [
				if 0 <> rs: dbpt db a b [
					print "error putting data."
					halt
				]
				;			db-sync db 0
				rs: dbgt-b db a
				if b <> rs: to-string rs [
					prin ["Data mismatch:" b "<>" rs]
					halt
				]
			]
		]
		db-close db
		print [newline dbname "=> passed."]
	]
]

tst-03: does [
	error? try [delete %recno.db]
	dbg-print "Opening Recno db ..."
	db: dbrecno-open "recno.db"

	dbg-print "Putting in records."
	forall test-data [
		dbg-prin ["Placing" test-data/1 "at" index? test-data "..."]
		rs: rec-put db index? test-data test-data/1
		dbg-print either 0 <> rs ["[err?]"]["ok"]
	] test-data: head test-data

	dbg-print "Syncing data.."

	db-sync db 0
			
	dbg-print "Getting records."
	repeat i length? test-data [
		dbg-prin ["Getting" i "..."]
		dt: rec-get db i
		either dt/data <> test-data/:i [
			dbg-print "***MISMATCH!***"
			probe dt
			dbg-print ["ORIG:" test-data/:i]
		][dbg-print "ok"]
	] 
	dbg-print "Closing recno db"
	db-close db
]

tst-01 
tst-02
;;-- Recno DB not finished
;tst-03