/*
**  DB Library REBOL interface
**  This is compiled against Berkely db-1.85 
**
**  Author:    Jeff Kreis
**  Date:      15-Jan-2002
**	Copyright: All rights reserved.
*/

#include "compat.h"
#include <sys/types.h>
#include <limits.h>
#include "../db-1.85/include/db.h"
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>


#ifdef WIN32
#include <windows.h>
#define LIBDB_API __declspec(dllexport)
#else
#define LIBDB_API 
#endif

/*
**  Global DBT
**  Holds results for us.
**  (Only okay for single threaded).
*/
DBT g_dbt;

/*
** Structs holding various constants
** used by DB, with accessor funcs
** to pass them back to REBOL
*/
struct T_dbflags {
	int a;
	int b;
	int c;
	int d;
	int e;
	int f;
	int g;
	int h;
	int i;
}  dbflags = {
	O_CREAT,
	O_EXCL,
	O_EXLOCK,
	O_NONBLOCK,
	O_RDONLY,
	O_RDWR,
	O_SHLOCK,
	O_TRUNC,
#if defined(WIN32)
	O_BINARY
#else 
	0
#endif
};
LIBDB_API struct T_dbflags* dbflg() {return &dbflags;}

LIBDB_API struct T_dbtypes {
	int a;
	int b;
	int c;
} dbtypes = {
	DB_BTREE,
	DB_HASH,
	DB_RECNO
};
LIBDB_API struct T_dbtypes* dbtyp() {return &dbtypes;}

struct T_dbcursors {
	int a;
	int b;
	int c;
	int d;
	int e;
	int f;
	int g;
	int h;
	int i;
} dbcursors = {
	R_CURSOR,
	R_IAFTER,
	R_IBEFORE,
	R_NOOVERWRITE,
	R_SETCURSOR,
	R_FIRST,
	R_LAST,
	R_NEXT,
	R_PREV
};
LIBDB_API struct T_dbcursors* dbcrs() {return &dbcursors;}

/*
** Some funcs that simplify the DB interface. 
*/
LIBDB_API DB* rdb_open(char *name, int flags, int mode, DBTYPE type) {
	return dbopen(name, flags, mode, type, NULL);
}

LIBDB_API DB* rdb_recno_open(char *name, int flags, int mode) {
	RECNOINFO rf = { R_NOKEY, 0, 0, 0, 0, '\0', NULL };
	return dbopen(name, flags, mode, DB_RECNO, &rf);
}

LIBDB_API int rdb_recno_put(DB *db, double loc, DBT* data, u_int flags) {
	DBT key;
	unsigned int uloc = (unsigned int) loc;
	key.data = &uloc;
	key.size = sizeof(unsigned int);
	return db->put(db, &key, data, flags);
}

LIBDB_API int rdb_recno_get(DB *db, double loc, DBT* data, u_int flags) {
	DBT key;
	unsigned int uloc = (unsigned int) loc;
	key.data = &uloc;
	key.size = sizeof(unsigned int);
	
	return db->get(db, &key, data, flags);
}

LIBDB_API int rdb_close(DB *db) { return db->close(db);}
LIBDB_API int rdb_fd(DB *db)    { return db->fd(db);   }

LIBDB_API int rdb_del(DB *db, DBT *key, u_int flags) {
	return db->del(db, key, flags);
}

LIBDB_API int rdb_get(DB *db, DBT *key, DBT *data, u_int flags) {
	return db->get(db, key, data, flags);
}

LIBDB_API int rdb_put(DB *db, DBT *key, DBT *data, u_int flags) {
	return db->put(db, key, data, flags);
}

LIBDB_API int rdb_seq(DB *db, DBT *key, DBT *data, u_int flags) {
	return db->seq(db, key, data, flags);
}

LIBDB_API int rdb_sync(DB *db, u_int flags) {
	return db->sync(db, flags);
}


/*
**  RDB_BGET_SIZE
**  To fetch an arbitrary binary 
**  record (8 bit clean):
**
**  Do a db get into the global DBT
**  and then return the size.  REBOL
**  side then creates appropriate 
**  sized binary! and passes that to
**  the binary get function. (See README.txt)
*/ 
LIBDB_API int rdb_bget_size(DB *db, DBT *key) {
	if(0 != db->get(db, key, &g_dbt, 0)) {
		return -1;
	}
	return g_dbt.size;
}

/*
**  RDB_BGET
**
**  After calling RDB_BGET_SIZE, 
**  we call this to actually fetch
**  the record.  We copy the binary 
**  data from the global DBT into 
**  the supplied memory location 
**  (See README.txt)
*/
LIBDB_API void rdb_bget(char *data) {
	memcpy(data, g_dbt.data, g_dbt.size);
}

/*
** Windows DLL 
*/
 
#ifdef WIN32

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					   )
{
	switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}

#endif
